/***************************************************************************
*  T B f f o l d e r                                                       *
*                                                                          *
*  Author..: Torsten Johann                                                *
*  E-mail..: t-johann@gmx.de                                               *
*  Date....: 13. Jan. 2000                                                 *
*  Version.: 1.00                                                          *
*  Status..: FREEWARE                                                      *
*  Function: Implementation of Win32 API function 'SHBrowseForFolder'      *
*            in Borland C++Builder 3.0                                     *
*                                                                          *
*  (C) Copyright 2000 Torsten Johann                                       *
*                                                                          *
***************************************************************************/

//---------------------------------------------------------------------------
#ifndef BffolderH
#define BffolderH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
#include <ShlObj.h>
#include <Shlwapi.h>

enum TBffRootDir {
rdDesktop,
rdInternet,
rdPrograms,
rdControlPanel,
rdPrinters,
rdMyDocuments,
rdFavorites,
rdStartup,
rdRecent,
rdSendTo,
rdRecycleBin,
rdStartMenu,
rdDesktopDirectory,
rdMyComputer,
rdNetwork,
rdNetHood,
rdFonts,
rdTemplates,
rdCommonStartMenu,
rdCommonPrograms,
rdCommonStartup,
rdCommonDesktopDirectory,
rdAppData,
rdPrintHood,
rdAltStartup,
rdCommonAltStartup,
rdCommonFavorites,
rdInternetCache,
rdCookies,
rdHistory,
rdUseSelectedDir
};

enum BffolderOption {bffBrowseForComputer, bffBrowseForPrinter,
                     bffDontGoBelowDomain, bffReturnFSAncestors,
                     bffReturnOnlyFSDirs, bffStatusText,
                     bffValidate, bffEditBox, bffBrowseIncludeFiles };
typedef Set <BffolderOption, bffBrowseForComputer, bffBrowseIncludeFiles>
            TBffOptions;

typedef void __fastcall
  (__closure *TSelChangedEvent)(System::TObject *Sender, AnsiString NewSel);

typedef void __fastcall
  (__closure *TValidateEvent)(System::TObject *Sender, AnsiString NewSel, bool& Continue);

#define PACKVERSION(major,minor) MAKELONG(minor,major)

//---------------------------------------------------------------------------
class PACKAGE TBffolder : public TCommonDialog
{
private:
    TBffOptions      FOptions;
    TBffRootDir      FRootDir;
    BROWSEINFO       bi;
    LPITEMIDLIST     ItemID;
    LPMALLOC         lpMalloc;
    TNotifyEvent     FOnInitialized;
    TSelChangedEvent FOnSelChanged;
    TValidateEvent   FOnValidateFailed;
    AnsiString       FTitle;
    AnsiString       FSelectedDir;
    AnsiString       FCaption;
    AnsiString       FDisplayName;
    AnsiString       FStatusText;
    bool             FEnableOK;
    void __fastcall SetCaption (const AnsiString ACaption);
    void __fastcall SetSelectedDir (const AnsiString AName);
    void __fastcall SetStatusText (const AnsiString AStatusText);
    void __fastcall SetEnableOK (bool AEnable);
    void __fastcall GetDllVersion (void);
    AnsiString _fastcall GetCaption (void);
protected:
public:
    __fastcall TBffolder(TComponent* Owner);
    bool __fastcall Execute (void);
    HWND   FhWnd;
    UINT   FMsg;
    LPARAM FlParam;
    LPARAM FlpData;
    DWORD  FDllVersion;
    LPSHELLFOLDER lpShellFolder;
    __property AnsiString DisplayName = {read=FDisplayName, write=FDisplayName};
    __property DWORD DllVersion = {read=FDllVersion};
__published:
    __property TNotifyEvent OnInitialized = {read=FOnInitialized, write=FOnInitialized};
    __property TSelChangedEvent OnSelectionChanged = {read=FOnSelChanged, write=FOnSelChanged};
    __property TValidateEvent OnValidateFailed = {read=FOnValidateFailed, write=FOnValidateFailed};
    __property AnsiString Title = {read=FTitle, write=FTitle};
    __property AnsiString SelectedDir = {read=FSelectedDir, write=SetSelectedDir};
    __property AnsiString Caption = {read=GetCaption, write=SetCaption};
    __property AnsiString StatusText = {read=FStatusText, write=SetStatusText};
    __property bool EnableOK = {read=FEnableOK, write=SetEnableOK, default=true};
    __property TBffOptions Options = {read=FOptions, write=FOptions, stored=true, default=bffStatusText};
    __property TBffRootDir RootDir = {read=FRootDir, write=FRootDir, default=rdDesktop};
};
//---------------------------------------------------------------------------
#endif
